//------------------------------------------------------------------------------
//                         Copyright 2003, P. Hulshoff
//                               The Netherlands
//------------------------------------------------------------------------------
//    This file is part of MAMED.
//
//    MAMED is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MAMED is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MAMED; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                                Includes
//------------------------------------------------------------------------------


#include "classes.h"
#include <iostream>
using namespace std;


//------------------------------------------------------------------------------
//                         Global class variables
//------------------------------------------------------------------------------


order_type order = category;


//------------------------------------------------------------------------------
//                           External variables
//------------------------------------------------------------------------------


extern string catver;


//------------------------------------------------------------------------------
//                                 Classes
//------------------------------------------------------------------------------


// Create a new game
game::game( string gname, string gfile, string gyear, string gclone_of )
{
  name      = gname;
  file      = gfile;
  year      = gyear;
  clone_of  = gclone_of;
  version   = "";
  cat       = "";
  orig_cat  = "";
  exec      = "";
  available = false;
  prev      = NULL;
  next      = NULL;
}


// Get the name of a game
string game::get_name( )
{
  return name;
}


// Get the filename of a game
string game::get_file( )
{
  return file;
}


// Get the year of a game
string game::get_year( )
{
  return year;
}


// Get the original version of a game
string game::get_clone_of( )
{
  return clone_of;
}


// Get the version of a game
string game::get_version( )
{
  return version;
}


// Set the version of a game
void game::set_version( string gversion )
{
  version = gversion;
}


// Get the category of a game
string game::get_cat( )
{
  return cat;
}


// Set the category of a game
void game::set_cat( string gcat )
{
  cat = gcat;
}


// Get the original category of a game
string game::get_orig_cat( )
{
  return orig_cat;
}


// Set the original category of a game
void game::set_orig_cat( string gcat )
{
  orig_cat = gcat;
}


// Is the game available?
bool game::is_available( )
{
  return available;
}


// Set game available
void game::set_available( bool avail )
{
  available = avail;
}


// Get the next game in the list
game *game::get_next( )
{
  return next;
}


// Set the next game in the list
void game::set_next( game *ggame )
{
  next = ggame;
}


// Get the previous game in the list
game *game::get_prev( )
{
  return prev;
}


// Set the previous game in the list
void game::set_prev( game *ggame )
{
  prev = ggame;
}


// Set the executable of a game
void game::set_exec( string gexec )
{
  exec = gexec;
}


// Get the executable of a game
string game::get_exec( )
{
  return exec;
}


// < operator on games
const bool operator< ( game &left, game &right )
{
  switch ( order )
  {
    case none:
      return (left.get_name( ) < right.get_name( ));
      break;
    case year:
      return (  (left.get_year( ) <  right.get_year( )) ||
               ((left.get_year( ) == right.get_year( )) &&
                (left.get_name( ) <  right.get_name( ))) );
      break;
    case version:
      return (  (left.get_version( ) <  right.get_version( )) ||
               ((left.get_version( ) == right.get_version( )) &&
                (left.get_name( )    <  right.get_name( ))) );
      break;
    case category:
      string catleft  = left.get_cat( );
      string catright = right.get_cat( );
      if ( catver == "stripped" )
      {
        string::size_type pos;
        if ( (pos = catleft.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
          catleft = catleft.substr( 0, pos-1 );
        if ( (pos = catleft.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
          catleft = catleft.substr( 0, pos );
        if ( (pos = catright.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
          catright = catright.substr( 0, pos-1 );
        if ( (pos = catright.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
          catright = catright.substr( 0, pos );
      }

      return (  (catleft          <  catright) ||
               ((catleft          == catright) &&
                (left.get_name( ) <  right.get_name( ))) );
      break;
  }
  return false;
}


// Sort the list of games.
// It expects list either to point to the start of a list, or to have the list be circular
game *sort( game *list, int nr_games )
{

  // Error situation
  if ( list == NULL || nr_games == 0 )
    return NULL;

  // Default situation
  if ( nr_games == 1 )
  {
    (*list).set_prev( list );
    (*list).set_next( list );
    return list;
  }

  // Define left list
  game *left = list;
  if ( (*left).get_prev( ) != NULL )
  {
    (*((*left).get_prev( ))).set_next( NULL );
    (*left).set_prev( NULL );
  }

  // Define right list
  game *right = list;
  for ( int i = 1; i <= nr_games/2; i++ )
    right = (*right).get_next( );
  if ( (*right).get_prev( ) != NULL )
  {
    (*((*right).get_prev( ))).set_next( NULL );
    (*right).set_prev( NULL );
  }

  // Sort the lists
  left  = sort( left, nr_games/2 );
  right = sort( right, nr_games-nr_games/2 );

  // Disconnect the lists
  (*((*left).get_prev( ))).set_next( NULL );
  (*left).set_prev( NULL );
  (*((*right).get_prev( ))).set_next( NULL );
  (*right).set_prev( NULL );

  // Resulting gamelist
  game *result;

  if ( (*left) < (*right) )
  {
    result = left;
    left = (*left).get_next( );
  }
  else
  {
    result = right;
    right = (*right).get_next( );
  }
  
  game *loop_game = result;
  while ( left != NULL || right != NULL )
  {
    if ( right == NULL || 
        (left  != NULL &&
        (*left) < (*right)) )
    {
      (*loop_game).set_next( left );
      (*left).set_prev( loop_game );
      loop_game = (*loop_game).get_next( );
      left      = (*left).get_next( );
    }
    else
    {
      (*loop_game).set_next( right );
      (*right).set_prev( loop_game );
      loop_game = (*loop_game).get_next( );
      right     = (*right).get_next( );
    }
  }  

  // Reconnect the first and last game
  (*loop_game).set_next( result );
  (*result).set_prev( loop_game );

  return result;
}
