//------------------------------------------------------------------------------
//                         Copyright 2003, P. Hulshoff
//                               The Netherlands
//------------------------------------------------------------------------------
//    This file is part of MAMED.
//
//    MAMED is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MAMED is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MAMED; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                                Includes
//------------------------------------------------------------------------------


#include <stdio.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <sys/types.h>
#include <dirent.h>
#include <fnmatch.h>
#include "classes.h"
#include "fileio.h"
#include "gui.h"
#include <unistd.h>
using namespace std;


//------------------------------------------------------------------------------
//                          MAMEd version string
//------------------------------------------------------------------------------


const string mamed_version = "MAMEd V0.7";


//------------------------------------------------------------------------------
//                            Global variables
//------------------------------------------------------------------------------


game          *current_game;
int            nr_games;
bool           play_music;
string         bg_image_name;
string         help_image_name;
int            help_image_x;
int            help_image_y;
int            help_image_alpha;
string         dir_mame;
string         dir_fonts;
vector<string> music_names;
string         music_dir;
bool           music_repeat;
int            nr_music;
string         mame_exec;
string         instructions;
int            screenx;
int            screeny;
int            screenc;
int            screenshotx;
int            screenshoty;
int            screenshotw;
int            screenshoth;
int            nr_print_games;
string         games_font;
int            games_font_size;
int            games_sel_font_size;
int            games_x;
int            games_y;
int            games_h_align;
int            games_v_align;
int            games_red;
int            games_green;
int            games_blue;
int            games_sel_red;
int            games_sel_green;
int            games_sel_blue;
string         category_font;
int            category_font_size;
int            category_x;
int            category_y;
int            category_h_align;
int            category_v_align;
int            category_red;
int            category_green;
int            category_blue;
int            category_sel_red;
int            category_sel_green;
int            category_sel_blue;
string         year_font;
int            year_font_size;
int            year_x;
int            year_y;
int            year_h_align;
int            year_v_align;
int            year_red;
int            year_green;
int            year_blue;
int            year_sel_red;
int            year_sel_green;
int            year_sel_blue;
string         version_font;
int            version_font_size;
int            version_x;
int            version_y;
int            version_h_align;
int            version_v_align;
int            version_red;
int            version_green;
int            version_blue;
int            version_sel_red;
int            version_sel_green;
int            version_sel_blue;
int            key_favourites;
int            key_music;
int            key_order;
int            key_play;
int            key_quit;
int            key_shot;
int            key_screendump;
int            key_next_game;
int            key_next_letter;
int            key_next_section;
int            key_prev_game;
int            key_prev_letter;
int            key_prev_section;
int            key_help;
display_type   shot = screenshot;
display_type   show_shot = screenshot;
int            screensaver_tba;
int            screensaver_tbs;
string         catver;
string         mature;
string         fadeout;


//------------------------------------------------------------------------------
//                           External variables
//------------------------------------------------------------------------------


extern order_type   order;
extern SDL_Surface *screen;


//------------------------------------------------------------------------------
//                                Functions
//------------------------------------------------------------------------------


// Read the file "gamelist" and generate an alphabetical list of games
void read_gamelistfile( )
{

  nr_games = 0;

  cout << "Reading gamelist file, and create list of games." << endl;
  SDL_Color color = {255, 255, 255, 0};
  print_string( "Generating MAMEd gamelist file",  screen, games_font, screen->w/30, screen->w/2, screen->h/2-screen->w/60, ALIGN_CENTER, ALIGN_CENTER, color );
  print_string( "This may take a while...",  screen, games_font, screen->w/30, screen->w/2, screen->h/2+screen->w/60, ALIGN_CENTER, ALIGN_CENTER, color );
  SDL_UpdateRect( screen, 0, 0, 0, 0 );

  FILE *gamelist_file;

  if ((gamelist_file = popen( (mame_exec + " -nolcf -listxml").c_str( ), "r" )) == NULL)
  {
    cout << "Cannot get game list information from MAME." << endl;
    cout << "Are you sure the " + mame_exec + " can be executed?" << endl;
    exit( 1 );
  }

  char           char_buf[BUFSIZ];
  vector<string> gamelist;

  while ( fgets( char_buf, sizeof ( char_buf ), gamelist_file ) )
  {
    string string_buf = char_buf;
    gamelist.push_back( string_buf );
  }

  bool games_found = false;

  for ( uint i = 0; i < gamelist.size( ); i++)
  {
    if ( gamelist[i].find( "game name=", 0 ) != string::npos ) // Game found
    {

      string::size_type pos1 = gamelist[i].find( "game name=\"", 0 ) + 11;
      string::size_type pos2 = gamelist[i].find( "\"", pos1 ) - pos1;
      string gfile = gamelist[i].substr( pos1, pos2 );

      string gclone_of = "";
      if ( gamelist[i].find( "cloneof=\"", 0 ) != string::npos )
      {
        string::size_type pos1 = gamelist[i].find( "cloneof=\"", 0 ) + 9;
        string::size_type pos2 = gamelist[i].find( "\"", pos1 ) - pos1;
        gclone_of = gamelist[i].substr( pos1, pos2 );
      }

      while ( gamelist[i].find( "<description>", 0 ) == string::npos ) // Find the description
        i++;

      pos1 = gamelist[i].find( "<description>", 0 ) + 13;
      pos2 = gamelist[i].find( "</description>", pos1 ) - pos1;
      string gname = gamelist[i].substr( pos1, pos2 );
      // Change &amp into & in the descriptions
      while ( gname.find( "&amp", 0 ) != string::npos )
        gname = gname.substr( 0, gname.find( "&amp", 0 ) + 1 ) +
                gname.substr( gname.find( "&amp", 0 ) + 5, string::npos );

      bool year_found = true;
      while ( gamelist[i].find( "<year>", 0 ) == string::npos ) // Find the year
      {
        if ( gamelist[i].find( "game name=", 0 ) != string::npos )
        {
          year_found = false;
          break;
        }
        i++;
      }

      string gyear;
      if ( year_found )
      {
        pos1 = gamelist[i].find( "<year>", 0 ) + 6;
        pos2 = gamelist[i].find( "</year>", pos1 ) - pos1;
        gyear = gamelist[i].substr( pos1, pos2 );
      }
      else
        gyear = "----";

      // Generate new game, and add it to the list
      game *new_game = new game( gname, gfile, gyear, gclone_of );
      string check_file = dir_mame + "/roms/" + gfile + ".zip";
//SDL_FillRect( screen, 0, 0 );
//print_string( check_file,  screen, games_font, screen->w/30, screen->w/2, screen->h/2+screen->w/60, ALIGN_CENTER, ALIGN_CENTER, color );
//SDL_UpdateRect( screen, 0, 0, 0, 0 );
//sleep(1);
//cout << "checking file" << endl;
      if ( access( check_file.c_str( ), 0 ) == 0 )
      {
//cout << "file found" << endl;
        (*new_game).set_available( true );
        games_found = true;
      }
      nr_games++;
      if ( current_game == NULL )
        current_game = new_game;
      else
      {
        (*new_game).set_prev( current_game );
        (*current_game).set_next( new_game );
        current_game = new_game;
      }

    }
  }

cout << "checked " << nr_games << " games" << endl;

  if ( nr_games == 0 || !games_found )
  {
    cout << "No games found in the MAME list information." << endl;
    exit( 1 );
  }

  return;

}


// Read the file "catver.ini", and insert the game categories into the gamelist
void read_catverfile( )
{

  string    line;
  string    catver_string = getenv( "HOME" );
  catver_string += "/.mamed/catver.ini";
  ifstream  fcatver( catver_string.c_str( ) );

  cout << "Reading catver.ini, and inserting game categories in the list of games." << endl;

  // Put current game at the start of the list
  while ( (*current_game).get_prev( ) != NULL )
    current_game = (*current_game).get_prev( );

  while ( getline( fcatver, line ) )
  {
    if ( line.find( "=", 0 ) != string::npos ) // strip unused lines
    {

      // Get filename
      string::size_type pos = line.find( "=", 0 );
      string gfile = line.substr( 0, pos );
      line.erase( 0, ++pos );

      // Get gametype
      string gtype;
      if ( (pos = line.find( "\r", 0 )) != string::npos ) // Remove trailing endlines
        gtype = line.substr( 0, pos );
      else
        gtype = line;
      

      // Set gametype to the game

      for ( game *loop_game = current_game; loop_game != NULL; loop_game = (*loop_game).get_next( ) )
        if ( (*loop_game).get_file( ) == gfile )
        {
          if ( (*loop_game).get_cat( ) == "" )
          {
            (*loop_game).set_cat( gtype );
            (*loop_game).set_orig_cat( gtype );
          }
          else
            (*loop_game).set_version( "0" + gtype );
        }
    }
  }

  // Filter mature games
  if ( mature == "off" )
    for ( game *loop_game = current_game; loop_game != NULL; loop_game = (*loop_game).get_next( ) )
      if ( ((*loop_game).get_cat( )).find( "Mature", 0 ) != string::npos )
        (*loop_game).set_available( false );

  fcatver.close( );

  return;

}


// Read the mamed gameslist
void read_mamedgamelistfile( )
{

  string   games_string = getenv( "HOME" );
  games_string += "/.mamed/mamed.games";
  ifstream fgames_list( games_string.c_str( ) );
  string   line;
  game    *first_game;
  bool     games_found = false;

  cout << "Reading mamed gameslist file and creating list of games." << endl;

  getline( fgames_list, line );

  // If new MAMEd version, try to retain some settings from the old mamed.games file
  if ( line != mamed_version )
  {
    read_gamelistfile( );
    read_catverfile( );
    getline( fgames_list, line ); // Order
    getline( fgames_list, line ); // Image shot

    // Retain Favourites category
    while ( getline ( fgames_list, line ) )
    {

      // Get filename
      string::size_type pos = line.find( "", 0 );
      string gfile = line.substr( 0, pos );
      line.erase( 0, ++pos );

      // Skip gamename, year, and original game
      for ( int i = 0; i < 3; i++ )
      {
        pos = line.find( "", 0 );
        line.erase( 0, ++pos );
      }

      // Get category
      pos = line.find( "", 0 );
      string cat = line.substr( 0, pos );
      line.erase( 0, ++pos );

      // Set category to Favourites for the games that used to have this category
      if ( cat == "Favourites" )
        for ( game *loop_game = (*current_game).get_next( ); loop_game != NULL; loop_game = (*loop_game).get_next( ) )
          if ( (*loop_game).get_file( ) == gfile )
            (*loop_game).set_cat( "Favourites" );
      
    }
    fgames_list.close( );
    return;
  }

  if ( getline( fgames_list, line ) )
  {
    if ( line == "none" )
      order = none;
    else if ( line == "year" )
      order = year;
    else if ( line == "version" )
      order = version;
    else
      order = category;
  }

  if ( getline( fgames_list, line ) )
  {

    if ( line == "screenshot" )
      shot = screenshot;
    else if ( line == "titleshot" )
      shot = titleshot;
    else if ( line == "flyer" )
      shot = flyer;
    else if ( line == "cabinet" )
      shot = cabinet;
    else if ( line == "marquee" )
      shot = marquee;
    else
      shot = cycle;

    if ( shot != cycle )
      show_shot = shot;
    else
      show_shot = screenshot;

  }

  nr_games = 0;

  while ( getline( fgames_list, line ) )
  {

    // Get filename
    string::size_type pos = line.find( "", 0 );
    string gfile = line.substr( 0, pos );
    line.erase( 0, ++pos );

    // Get gamename
    pos = line.find( "", 0 );
    string gname = line.substr( 0, pos );
    line.erase( 0, ++pos );
    
    // Get year
    pos = line.find( "", 0 );
    string gyear = line.substr( 0, pos );
    line.erase( 0, ++pos );

    // Get original game
    pos = line.find( "", 0 );
    string gclone_of = line.substr( 0, pos );
    line.erase( 0, ++pos );

    // Generate new game, and add it to the list
    game *new_game = new game( gname, gfile, gyear, gclone_of );
    string check_file = dir_mame + "/roms/" + gfile + ".zip";
    if ( access( check_file.c_str( ), 0 ) == 0 )
    {
      (*new_game).set_available( true );
      games_found = true;
    }
    nr_games++;
    if ( current_game == NULL )
    {
      current_game = new_game;
      first_game   = current_game;
    }
    else
    {
      (*new_game).set_prev( current_game );
      (*current_game).set_next( new_game );
      current_game = new_game;
    }

    // Get category
    pos = line.find( "", 0 );
    string cat = line.substr( 0, pos );
    line.erase( 0, ++pos );
    (*new_game).set_cat( cat );

    // Get original category
    pos = line.find( "", 0 );
    string orig_cat = line.substr( 0, pos );
    line.erase( 0, ++pos );
    (*new_game).set_orig_cat( orig_cat );

    // Get version
    pos = line.find( "", 0 );
    string version = line.substr( 0, pos );
    line.erase( 0, ++pos );
    (*new_game).set_version( version );

    (*new_game).set_exec( line );
    if ( (*new_game).get_exec( ) != "" )
    {
      (*new_game).set_available( true );
      games_found = true;
    }

  }

  if ( nr_games == 0 || !games_found )
  {
    cout << "No games found in gamelist file " << fgames_list << endl;
    exit( 1 );
  }

  // Put current game at the start of the list
  game *last_game = current_game;
  while ( (*current_game).get_prev( ) != NULL )
    current_game = (*current_game).get_prev( );

  // Filter mature games
  if ( mature == "off" )
    for ( game *loop_game = current_game; loop_game != NULL; loop_game = (*loop_game).get_next( ) )
      if ( ((*loop_game).get_cat( )).find( "Mature", 0 ) != string::npos )
        (*loop_game).set_available( false );

  // Connect the first and last game
  (*last_game).set_next( current_game );
  (*current_game).set_prev( last_game );

  // Set current game to the first game in the file
  current_game = first_game;

  fgames_list.close( );

  return;

}


// Write the mamed gameslist
void write_mamedgamelistfile( )
{

  string   games_string = getenv( "HOME" );
  games_string += "/.mamed/mamed.games";
  ofstream fgames_list( games_string.c_str( ) );
  string   line;

  fgames_list << mamed_version << endl;
  if ( order == none )
    fgames_list << "none" << endl;
  else if ( order == year )
    fgames_list << "year" << endl;
  else if ( order == version )
    fgames_list << "version" << endl;
  else
    fgames_list << "category" << endl;

  if ( shot == screenshot )
    fgames_list << "screenshot" << endl;
  else if ( shot == titleshot )
    fgames_list << "titleshot" << endl;
  else if ( shot == flyer )
    fgames_list << "flyer" << endl;
  else if ( shot == cabinet )
    fgames_list << "cabinet" << endl;
  else if ( shot == marquee )
    fgames_list << "marquee" << endl;
  else
    fgames_list << "cycle" << endl;

  fgames_list << (*current_game).get_file( )     << ""
              << (*current_game).get_name( )     << ""
              << (*current_game).get_year( )     << ""
              << (*current_game).get_clone_of( ) << ""
              << (*current_game).get_cat( )      << ""
              << (*current_game).get_orig_cat( ) << ""
              << (*current_game).get_version( )  << ""
              << (*current_game).get_exec( )     << endl;

  for ( game *loop_game = (*current_game).get_next( ); loop_game != current_game; loop_game = (*loop_game).get_next( ) )
    fgames_list << (*loop_game).get_file( )     << ""
                << (*loop_game).get_name( )     << ""
                << (*loop_game).get_year( )     << ""
                << (*loop_game).get_clone_of( ) << ""
                << (*loop_game).get_cat( )      << ""
                << (*loop_game).get_orig_cat( ) << ""
                << (*loop_game).get_version( )  << ""
                << (*loop_game).get_exec( )     << endl;

  fgames_list.close( );

  return;

}


// Read the gamelists "gamelist" and "catver.ini" if the mamed games file is
// not available, and generate the gamelist
void read_gamelist( )
{

  string games_string = getenv( "HOME" );
  games_string += "/.mamed/mamed.games";
  ifstream games_list( games_string.c_str( ) );

  if ( games_list == NULL )
  {
    read_gamelistfile( );
    read_catverfile( );
  }
  else
  {
    games_list.close( );
    read_mamedgamelistfile( );
  }

  // Sort the gamelist
  sort( current_game, nr_games );

  // Set current_game to an available game
  while ( !(*current_game).is_available( ) )
    current_game = (*current_game).get_next( );

  cout << "List of games created." << endl;

  return;

}


// Read the mamed.keyconfig configuration file
void read_key_config( )
{

  string    keycfg_string = getenv( "HOME" );
  keycfg_string += "/.mamed/mamed.keycfg";
  ifstream  keycfg_file( keycfg_string.c_str( ) );
  string    line;

  bool      config_found = false;

  cout << "Reading key configuration." << endl;

  while ( getline( keycfg_file, line ) )
  {
    config_found = true;
    string::size_type pos = line.find( " ", 0 );
    string word = line.substr( 0, pos );
    line.erase( 0, ++pos );
    while ( line[0] == ' ' )
      line.erase( 0, 1 );
    if ( word == "key_favourites" )
      key_favourites = atoi( line.c_str( ) );
    if ( word == "key_music" )
      key_music = atoi( line.c_str( ) );
    if ( word == "key_order" )
      key_order = atoi( line.c_str( ) );
    if ( word == "key_play" )
      key_play = atoi( line.c_str( ) );
    if ( word == "key_quit" )
      key_quit = atoi( line.c_str( ) );
    if ( word == "key_shot" )
      key_shot = atoi( line.c_str( ) );
    if ( word == "key_screendump" )
      key_screendump = atoi( line.c_str( ) );
    if ( word == "key_next_game" )
      key_next_game = atoi( line.c_str( ) );
    if ( word == "key_next_letter" )
      key_next_letter = atoi( line.c_str( ) );
    if ( word == "key_next_section" )
      key_next_section = atoi( line.c_str( ) );
    if ( word == "key_prev_game" )
      key_prev_game = atoi( line.c_str( ) );
    if ( word == "key_prev_letter" )
      key_prev_letter = atoi( line.c_str( ) );
    if ( word == "key_prev_section" )
      key_prev_section = atoi( line.c_str( ) );
    if ( word == "key_help" )
      key_help = atoi( line.c_str( ) );
  }

  if ( !config_found )
  {
    cout << "No keyconfigurations found in key configuration file " << keycfg_string << endl;
    exit( 1 );
  }

  cout << "Keys are configured." << endl;

  return;

}


// Read the mamed.cfg configuration file
void read_config( )
{

  string    cfg_string = getenv( "HOME" );
  cfg_string += "/.mamed/mamed.cfg";
  ifstream  cfg_file( cfg_string.c_str( ) );
  string    line;

  bool     config_found = false;

  cout << "Reading mamed configuration file." << endl;

  read_key_config( );

  while ( getline( cfg_file, line ) )
  {
    config_found = true;
    if ( line.find( "#", 0 ) == string::npos ) // strip comment lines
    {
      string::size_type pos = line.find( " ", 0 );
      string word = line.substr( 0, pos );
      line.erase( 0, ++pos );
      while ( line[0] == ' ' )
        line.erase( 0, 1 );
      if ( word == "screen" )
      {
        pos = line.find( ",", 0 );
        screenx = atoi( (line.substr( 0, pos)).c_str( )  );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        screeny = atoi( (line.substr( 0, pos)).c_str( )  );
        line.erase( 0, ++pos );
        screenc = atoi( line.c_str( ) );
      }
      if ( word == "screenshot" )
      {
        pos = line.find( ",", 0 );
        screenshotx = atoi( (line.substr( 0, pos)).c_str( )  );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        screenshoty = atoi( (line.substr( 0, pos)).c_str( )  );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        screenshotw = atoi( (line.substr( 0, pos)).c_str( )  );
        line.erase( 0, ++pos );
        screenshoth = atoi( line.c_str( ) );
      }
      if ( word == "bg_image" )
        bg_image_name = line;
      if ( word == "help_image" )
      {
        pos = line.find( ",", 0 );
        help_image_name = line.substr( 0, pos );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        help_image_x = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        help_image_y = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        help_image_alpha = atoi( line.c_str( ) );
      }
      if ( word == "screensaver" )
      {
        pos = line.find( ",", 0 );
        screensaver_tba = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        screensaver_tbs = atoi( line.c_str( ) );
      }
      if ( word == "music" )
      {
        pos = line.find( ",", 0 );
        nr_music = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        if ( nr_music == 0 )
          play_music = false;
        else
        {
          play_music = true;
          pos = line.find( ",", 0 );
          string repeat = line.substr( 0, pos );
          if ( repeat == "single" )
            music_repeat = false;
          else
            music_repeat = true;
          line.erase( 0, ++pos );
          music_dir = line;
          DIR *dir = opendir( music_dir.c_str( ) );
          struct dirent *e;
          if ( dir == NULL )
          {
            cout << "Cannot open music directory: " << music_dir << "." << endl;
            exit( 1 );
          }
          while ( (e = readdir( dir )) != NULL )
            if ( (fnmatch( "*.mp3", e->d_name, 0) == 0) ||
                 (fnmatch( "*.ogg", e->d_name, 0) == 0) )
              music_names.push_back( e->d_name );
        }
      }
      if ( word == "dir_mame" )
        dir_mame = line;
      if ( word == "dir_fonts" )
        dir_fonts = line;
      if ( word == "mame_exec" )
        mame_exec = line;
      if ( word == "instructions" )
        while ( getline( cfg_file, line ) )
        {
          if ( line == ";" ) // End of instructions
            break;
          instructions = instructions + line + "\n";
        }
      if ( word == "nr_games" )
        nr_print_games = atoi( (line.substr( 0, pos)).c_str( ) );
      if ( word == "print_game" )
      {
        string::size_type pos = line.find( ",", 0 );
        games_font = line.substr( 0, pos );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_font_size = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_sel_font_size = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_x = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_y = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        string word = line.substr( 0, pos );
        if ( word == "ALIGN_LEFT" )
          games_h_align = ALIGN_LEFT;
        else if ( word == "ALIGN_RIGHT" )
          games_h_align = ALIGN_RIGHT;
        else
          games_h_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        word = line.substr( 0, pos );
        if ( word == "ALIGN_TOP" )
          games_v_align = ALIGN_TOP;
        else if ( word == "ALIGN_BOTTOM" )
          games_v_align = ALIGN_BOTTOM;
        else
          games_v_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_sel_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        games_sel_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( "\n", 0 );
        games_sel_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
      }
      if ( word == "category" )
      {
        string::size_type pos = line.find( ",", 0 );
        category_font = line.substr( 0, pos );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_font_size = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_x = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_y = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        string word = line.substr( 0, pos );
        if ( word == "ALIGN_LEFT" )
          category_h_align = ALIGN_LEFT;
        else if ( word == "ALIGN_RIGHT" )
          category_h_align = ALIGN_RIGHT;
        else
          category_h_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        word = line.substr( 0, pos );
        if ( word == "ALIGN_TOP" )
          category_v_align = ALIGN_TOP;
        else if ( word == "ALIGN_BOTTOM" )
          category_v_align = ALIGN_BOTTOM;
        else
          category_v_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_sel_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        category_sel_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( "\n", 0 );
        category_sel_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
      }
      if ( word == "year" )
      {
        string::size_type pos = line.find( ",", 0 );
        year_font = line.substr( 0, pos );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_font_size = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_x = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_y = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        string word = line.substr( 0, pos );
        if ( word == "ALIGN_LEFT" )
          year_h_align = ALIGN_LEFT;
        else if ( word == "ALIGN_RIGHT" )
          year_h_align = ALIGN_RIGHT;
        else
          year_h_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        word = line.substr( 0, pos );
        if ( word == "ALIGN_TOP" )
          year_v_align = ALIGN_TOP;
        else if ( word == "ALIGN_BOTTOM" )
          year_v_align = ALIGN_BOTTOM;
        else
          year_v_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_sel_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        year_sel_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( "\n", 0 );
        year_sel_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
      }
      if ( word == "version" )
      {
        string::size_type pos = line.find( ",", 0 );
        version_font = line.substr( 0, pos );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_font_size = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_x = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_y = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        string word = line.substr( 0, pos );
        if ( word == "ALIGN_LEFT" )
          version_h_align = ALIGN_LEFT;
        else if ( word == "ALIGN_RIGHT" )
          version_h_align = ALIGN_RIGHT;
        else
          version_h_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        word = line.substr( 0, pos );
        if ( word == "ALIGN_TOP" )
          version_v_align = ALIGN_TOP;
        else if ( word == "ALIGN_BOTTOM" )
          version_v_align = ALIGN_BOTTOM;
        else
          version_v_align = ALIGN_CENTER;
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_sel_red = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( ",", 0 );
        version_sel_green = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
        pos = line.find( "\n", 0 );
        version_sel_blue = atoi( (line.substr( 0, pos )).c_str( ) );
        line.erase( 0, ++pos );
      }
      if ( word == "catver" )
        catver = line;
      if ( word == "mature" )
        mature = line;
      if ( word == "fadeout" )
        fadeout = line;
    }
  }

  if ( !config_found )
  {
    cout << "No configurations found in configuration file " << cfg_string << endl;
    exit( 1 );
  }

  cfg_file.close( );

  cout << "Mamed is configured." << endl;

  return;

}
