//------------------------------------------------------------------------------
//                         Copyright 2003, P. Hulshoff
//                               The Netherlands
//------------------------------------------------------------------------------
//    This file is part of MAMED.
//
//    MAMED is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MAMED is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MAMED; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                                Includes
//------------------------------------------------------------------------------


#include <iostream>
#include <string>
#include <vector>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <fnmatch.h>
#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_mixer.h"
#include "SDL/SDL_ttf.h"
#include "classes.h"
#include "fileio.h"
#include "gui.h"
using namespace std;


//------------------------------------------------------------------------------
//                            Global variables
//------------------------------------------------------------------------------


SDL_Surface *screen;
SDL_Surface *bg_image;
int          screensaver_timer;




//------------------------------------------------------------------------------
//                           External variables
//------------------------------------------------------------------------------


extern order_type     order;
extern int            screenx;
extern int            screeny;
extern int            screenc;
extern game          *current_game;
extern SDL_Surface   *screen;
extern SDL_Surface   *bg_image;
extern string         bg_image_name;
extern string         dir_mame;
extern string         dir_fonts;
extern bool           play_music;
extern string         instructions;
extern int            screenshotx;
extern int            screenshoty;
extern int            screenshotw;
extern int            screenshoth;
extern int            nr_print_games;
extern string         games_font;
extern int            games_font_size;
extern int            games_sel_font_size;
extern int            games_x;
extern int            games_y;
extern int            games_h_align;
extern int            games_v_align;
extern int            games_red;
extern int            games_green;
extern int            games_blue;
extern int            games_sel_red;
extern int            games_sel_green;
extern int            games_sel_blue;
extern string         category_font;
extern int            category_font_size;
extern int            category_x;
extern int            category_y;
extern int            category_h_align;
extern int            category_v_align;
extern int            category_red;
extern int            category_green;
extern int            category_blue;
extern int            category_sel_red;
extern int            category_sel_green;
extern int            category_sel_blue;
extern string         year_font;
extern int            year_font_size;
extern int            year_x;
extern int            year_y;
extern int            year_h_align;
extern int            year_v_align;
extern int            year_red;
extern int            year_green;
extern int            year_blue;
extern int            year_sel_red;
extern int            year_sel_green;
extern int            year_sel_blue;
extern string         version_font;
extern int            version_font_size;
extern int            version_x;
extern int            version_y;
extern int            version_h_align;
extern int            version_v_align;
extern int            version_red;
extern int            version_green;
extern int            version_blue;
extern int            version_sel_red;
extern int            version_sel_green;
extern int            version_sel_blue;
extern display_type   shot;
extern display_type   show_shot;
extern int            screensaver_tbs;
extern bool           screensaver;
extern bool           game_played;
extern bool           help_key_pressed;
extern string         help_image_name;
extern int            help_image_x;
extern int            help_image_y;
extern int            help_image_alpha;
extern string         catver;


//------------------------------------------------------------------------------
//                                Functions
//------------------------------------------------------------------------------


// Initiate the SDL screen, TTF fonts, and audio
void init_sdl( )
{
  if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO ) < 0 )
  {
    cout << "Cannot initialise SDL: " << SDL_GetError( ) << endl;
    exit( 1 );
  }
  SDL_ShowCursor( SDL_DISABLE );
  atexit( SDL_Quit );

  if ( (screen = SDL_SetVideoMode( screenx, screeny, screenc, SDL_SWSURFACE )) == NULL )
  {
    cout << "Cannot open screen: " << SDL_GetError( ) << endl;
    exit( 1 );
  }
  if ( (bg_image = IMG_Load( bg_image_name.c_str( ) )) == NULL )
  {
    cout << "Cannot open background image " << bg_image_name << endl;
    exit( 1 );
  }

  if ( TTF_Init( ) < 0 )
  {
    cout << "Cannot initialise TTF: " << SDL_GetError( ) << endl;
    exit( 1 );
  }
  atexit(TTF_Quit);

  int    audio_rate     = 44100;
  Uint16 audio_format   = AUDIO_S16;
  int    audio_channels = 2;
  int    audio_buffers  = 4096;

  if ( play_music )
    if ( Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0 )
    {
      cout << "Cannot open audio: " << SDL_GetError( ) << endl;
      play_music = false;
    }

  return;

}


// Print a string on the screen using a TTF font
void print_string( string name, SDL_Surface *screen, string fontname, int size, int x, int y, int h_align, int v_align, SDL_Color color )
{
  // Don't try to print empty strings
  if ( name == "" )
    return;

  TTF_Font    *font;
  if ( (font = TTF_OpenFont( (dir_fonts + "/" + fontname).c_str( ), size )) == NULL )
  {
    cout << "Cannot open font " << dir_fonts << "/" << fontname << endl;
    exit( 1 );
  }
  SDL_Surface *sText;
  if ( (sText = TTF_RenderText_Blended( font, name.c_str( ), color )) == NULL )
  {
    cout << "Cannot render text " << name << " using font " << font << endl;
    exit( 1 );
  }
  int w = 0;
  int h = 0;
  TTF_SizeText( font, name.c_str( ), &w, &h);
  SDL_Rect dstrect = {0, 0, 0, 0};
  SDL_Rect srcrect = {0, 0, w, h};

  // Horizontal alignment
  switch ( h_align )
  {
    case ALIGN_CENTER:
      dstrect.x = x-w/2;
      break;
    case ALIGN_RIGHT:
      dstrect.x = x-w;
      break;
    default:
      dstrect.x = x;
  }

  // Vertical alignment
  switch ( v_align )
  {
    case ALIGN_CENTER:
      dstrect.y = y-h/2;
      break;
    case ALIGN_BOTTOM:
      dstrect.y = y-h;
      break;
    default:
      dstrect.y = y;
  }

  // Prevent strings from being printed outside the screen
  if ( dstrect.x < screen->w/20 )
    dstrect.x = screen->w/20;
  if ( w > screen->w - screen->w/10 )
    srcrect.w = screen->w - screen->w/10;

  // Place text on screen
  if ( SDL_BlitSurface( sText, &srcrect, screen, &dstrect ) < 0 )
  {
    cout << "Cannot blit text " << name << " using font " << font << endl;
    exit( 1 );
  }
  TTF_CloseFont( font );
  SDL_FreeSurface( sText );

  return;

}


game        *print_game            = NULL;
game        *last_game             = NULL;
order_type   last_order            = none;
display_type last_shot             = screenshot;
string       last_category         = "";
bool         prev_help_key_pressed = false;

// Update the screen based on the current game
void update_screen( )
{
  if ( last_game        != current_game ||
       last_order       != order        ||
       last_shot        != show_shot    ||
       game_played                      ||
       help_key_pressed                 ||
       prev_help_key_pressed            ||
       last_category    != (*current_game).get_cat( ) ) // Check if update is needed
  {
    game_played           = false;
    prev_help_key_pressed = help_key_pressed;
    last_order            = order;
    last_shot             = show_shot;
    last_category         = (*current_game).get_cat( );
    last_game             = current_game;
    print_game            = current_game;

    // Clear screen
    if ( SDL_BlitSurface( bg_image, NULL, screen, NULL ) < 0 )
    {
      cout << "Could not blit the background image to the screen" << endl;
      exit( 1 );
    }

    SDL_Surface *displayshot;
    string       show_name;

    for (int i = 0; i < 5; i++ )
    {

      // Print screenshot
      string ssfname = dir_mame;
      if ( show_shot == screenshot )
        ssfname += "/snap/";
      else if ( show_shot == cabinet )
        ssfname += "/cabinets/";
      else if ( show_shot == flyer )
        ssfname += "/flyers/";
      else if ( show_shot == titleshot )
        ssfname += "/titles/";
      else
        ssfname += "/marquees/";
      show_name   = ssfname + (*current_game).get_file( ) + ".png";
      displayshot = IMG_Load( show_name.c_str( ) );

      if ( displayshot == NULL ) // Screenshot of current game not available
      {
        // Find original game, if available
        for ( game *loop_game = (*current_game).get_next( ); loop_game != current_game; loop_game = (*loop_game).get_next( ) )
          if ( (*loop_game).get_file( ) == (*current_game).get_clone_of( ) )
          {
            show_name = ssfname + (*loop_game).get_file( ) + ".png";
            displayshot = IMG_Load( show_name.c_str( ) );
          }
      }

      if ( displayshot == NULL && shot == cycle )
        switch ( show_shot )
        {
          case screenshot: show_shot = titleshot;break;
          case titleshot:  show_shot = flyer;break;
          case flyer:      show_shot = cabinet;break;
          case cabinet:    show_shot = marquee;break;
          case marquee:    show_shot = screenshot;break;
          case cycle:      show_shot = screenshot;break;
      }
      else
        break;

    }

    if ( displayshot != NULL )
    {

      // Use alpha blending to display the screenshot
      if ( SDL_SetAlpha( displayshot, SDL_RLEACCEL|SDL_SRCALPHA, 200 ) < 0 )
        cout << "Could not set alpha blending for screenshot " << show_name << endl;

      // Scale the screenshot to the screen
      float scale;
      if ( (float)displayshot->w/(float)screenshotw > (float)displayshot->h/(float)screenshoth )
        scale = (float)displayshot->w/(float)screenshotw;
      else
        scale = (float)displayshot->h/(float)screenshoth;
      SDL_Rect srcrect = { 0, 0, 1, 1 };
      SDL_Rect dstrect = {screenshotx-(int)((float)displayshot->w/scale)/2, screenshoty-(int)((float)displayshot->h/scale)/2,  1, 1 };
      float ycnt = 0.0;
      while ( srcrect.y < displayshot->h)
      {
        float xcnt = 0.0;
        ycnt++;
        while ( ycnt > scale ) // Check if we can print this line
        {
          ycnt -= scale;
          while ( srcrect.x < displayshot->w)
          {
            xcnt++;
            while ( xcnt > scale ) // Check if we can print this pixel
            {
              xcnt -= scale;
              if ( SDL_BlitSurface( displayshot, &srcrect, screen, &dstrect ) < 0 )
              {
                cout << "Cannot blit pixel of screenshot " << show_name << endl;
                exit ( 1 );
              }
              dstrect.x++;
            }
            srcrect.x++;
          }
          srcrect.x = 0;
          dstrect.x = screenshotx-(int)(float)(displayshot->w/scale)/2;
          dstrect.y++;
        }
        srcrect.y++;
      }

    }
    SDL_FreeSurface( displayshot );

    // Print instructions
    string tmp_instructions = instructions;
    string::size_type pos;
    while ( (pos = tmp_instructions.find( "\n", 0)) != string::npos )
    {
      string::size_type pos = tmp_instructions.find( ",", 0 );
      string text = tmp_instructions.substr( 0, pos );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      string font = tmp_instructions.substr( 0, pos );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      int size = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      int x = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      int y = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      string word = tmp_instructions.substr( 0, pos );
      int h_align;
      if ( word == "ALIGN_LEFT" )
        h_align = ALIGN_LEFT;
      else if ( word == "ALIGN_RIGHT" )
        h_align = ALIGN_RIGHT;
      else
        h_align = ALIGN_CENTER;
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      word = tmp_instructions.substr( 0, pos );
      int v_align;
      if ( word == "ALIGN_TOP" )
        v_align = ALIGN_TOP;
      else if ( word == "ALIGN_BOTTOM" )
        v_align = ALIGN_BOTTOM;
      else
        v_align = ALIGN_CENTER;
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      int red = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( ",", 0 );
      int green = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      pos = tmp_instructions.find( "\n", 0 );
      int blue = atoi( (tmp_instructions.substr( 0, pos )).c_str( ) );
      tmp_instructions.erase( 0, ++pos );
      SDL_Color color = {red, green, blue, 0};
      print_string( text, screen, font, size, x, y, h_align, v_align, color );
    }

    // We want to print nr_print_games/2 games before, and nr_games/2 games after the current game
    for ( int i = 1; i <= nr_print_games/2; i++ )
    {
      print_game = (*print_game).get_prev( );
      while ( !(*print_game).is_available( ) )
        print_game = (*print_game).get_prev( );
    }

    // Print gametitles
    for ( int i = 0; i < nr_print_games/2; i++ )
    {
      SDL_Color color = {games_red, games_green, games_blue, 0};
      print_string( (*print_game).get_name( ), screen, games_font, games_font_size, games_x,
                     games_y+i*3*games_font_size/2, games_h_align, games_v_align, color );
      print_game = (*print_game).get_next( );
      while ( !(*print_game).is_available( ) )
        print_game = (*print_game).get_next( );
    }
    SDL_Color color = {games_sel_red, games_sel_green, games_sel_blue, 0};
    print_string( (*print_game).get_name( ), screen, games_font, games_sel_font_size, games_x,
                   games_y+(nr_print_games/2-1)*3*games_font_size/2+3*(games_font_size+games_sel_font_size)/4,
                   games_h_align, games_v_align, color );
    print_game = (*print_game).get_next( );
    while ( !(*print_game).is_available( ) )
      print_game = (*print_game).get_next( );
    for ( int i = 0; i < (nr_print_games-1-nr_print_games/2); i++ )
    {
      SDL_Color color = {games_red, games_green, games_blue, 0};
      print_string( (*print_game).get_name( ), screen, games_font, games_font_size, games_x,
                     games_y+(nr_print_games/2+i)*3*games_font_size/2+3*games_sel_font_size/2,
                     games_h_align, games_v_align, color );
      print_game = (*print_game).get_next( );
      while ( !(*print_game).is_available( ) )
        print_game = (*print_game).get_next( );
    }

    // Print category
    if ( order == category )
    {
      SDL_Color color = {category_sel_red, category_sel_green, category_sel_blue, 0};
      string catprint = (*current_game).get_cat( );
      if ( catver == "stripped" )
      {
        if ( (pos = catprint.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
          catprint = catprint.substr( 0, pos-1 );
        if ( (pos = catprint.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
          catprint = catprint.substr( 0, pos );
      }
      print_string( catprint, screen, category_font, category_font_size, category_x, category_y, category_h_align, category_v_align, color );
    }
    else
    {
      SDL_Color color = {category_red, category_green, category_blue, 0};
      string catprint = (*current_game).get_cat( );
      if ( catver == "stripped" )
      {
        if ( (pos = catprint.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
          catprint = catprint.substr( 0, pos-1 );
        if ( (pos = catprint.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
          catprint = catprint.substr( 0, pos );
      }
      print_string( catprint, screen, category_font, category_font_size, category_x, category_y, category_h_align, category_v_align, color );
    }

    // Print gameyear
    if ( order == year )
    {
      SDL_Color color = {year_sel_red, year_sel_green, year_sel_blue, 0};
      print_string( (*current_game).get_year( ), screen, year_font, year_font_size, year_x, year_y, year_h_align, year_v_align, color );
    }
    else
    {
      SDL_Color color = {year_red, year_green, year_blue, 0};
      print_string( (*current_game).get_year( ), screen, year_font, year_font_size, year_x, year_y, year_h_align, year_v_align, color );
    }

    // Print gameversion
    if ( order == version )
    {
      SDL_Color color = {version_sel_red, version_sel_green, version_sel_blue, 0};
      print_string( (*current_game).get_version( ), screen, version_font, version_font_size, version_x, version_y, version_h_align, version_v_align, color );
    }
    else
    {
      SDL_Color color = {version_red, version_green, version_blue, 0};
      print_string( (*current_game).get_version( ), screen, version_font, version_font_size, version_x, version_y, version_h_align, version_v_align, color );
    }

    if ( help_key_pressed )
    {
      SDL_Surface *help_image = IMG_Load( help_image_name.c_str( ) );
      if ( help_image != NULL )
      {
        if ( SDL_SetAlpha( help_image, SDL_RLEACCEL|SDL_SRCALPHA, help_image_alpha ) < 0 )
          cout << "Could not set alpha blending for help image " << help_image_name << endl;
        SDL_Rect dstrect = {help_image_x-(int)((float)help_image->w)/2, help_image_y-(int)((float)help_image->h)/2,  help_image->w, help_image->h };
        if ( SDL_BlitSurface( help_image, NULL, screen, &dstrect ) < 0 )
        {
          cout << "Could not blit the help image to the screen" << endl;
          exit( 1 );
        }
        SDL_FreeSurface( help_image );
      }
    }

    // Update screen
    SDL_UpdateRect( screen, 0, 0, 0, 0 );

  }

  return;

}


// Run the screensaver
void run_screensaver( )
{

  if ( screensaver_timer == 0 )
  {

    // Clear screen
    if ( SDL_FillRect( screen, 0, 0 ) < 0 )
    {
      cout << "Cannot clear the screen: " << SDL_GetError( ) << endl;
      exit( 1 );
    }

    // Print screenshot
    string ssfname = dir_mame + "/snap/";
    DIR *dir = opendir( ssfname.c_str( ) );
    if ( dir == NULL )
    {
      cout << "Cannot open snapshot directory." << endl;
      exit(1);
    }
    vector<string> screenshot_names;
    struct dirent *e;
    while ( (e = readdir( dir )) != NULL )
      if (fnmatch( "*.png", e->d_name, 0) == 0)
        screenshot_names.push_back( e->d_name );

    for ( int x = 0; x < 3; x++ )
      for ( int y = 0; y < 3; y++ )
      {
        string show_name = ssfname + screenshot_names[(rand( )%screenshot_names.size( ))];
        SDL_Surface *screenshot = IMG_Load( show_name.c_str( ) );

        if ( screenshot != NULL )
        {
          // Scale the screenshot to the screen
          float scale;
          if ( (float)screenshot->w/((float)screen->w/3.0) > (float)screenshot->h/((float)screen->h/3.0) )
            scale = (float)screenshot->w/((float)screen->w/3.0);
          else
            scale = (float)screenshot->h/((float)screen->h/3.0);
          SDL_Rect srcrect = { 0, 0, 1, 1 };
          SDL_Rect dstrect = {(x*screen->w/3+screen->w/6)-(int)((float)screenshot->w/scale)/2, (y*screen->h/3+screen->h/6)-(int)((float)screenshot->h/scale)/2,  1, 1 };
          float ycnt = 0.0;
          while ( srcrect.y < screenshot->h)
          {
            float xcnt = 0.0;
            ycnt++;
            while ( ycnt > scale ) // Check if we can print this line
              {
            ycnt -= scale;
              while ( srcrect.x < screenshot->w)
              {
                xcnt++;
                while ( xcnt > scale ) // Check if we can print this pixel
                  {
                  xcnt -= scale;
                  if ( SDL_BlitSurface( screenshot, &srcrect, screen, &dstrect ) < 0 )
                  {
                    cout << "Cannot blit pixel of screenshot " << show_name << endl;
                    exit ( 1 );
                 }
                  dstrect.x++;
                }
                srcrect.x++;
              }
              srcrect.x = 0;
              dstrect.x = (x*screen->w/3+screen->w/6)-(int)(float)(screenshot->w/scale)/2;
              dstrect.y++;
            }
            srcrect.y++;
          }
        }
        SDL_FreeSurface( screenshot );
      }

    // Update screen
    SDL_UpdateRect( screen, 0, 0, 0, 0 );
    closedir(dir);

  }

  // Update screensaver timer
  screensaver_timer = (screensaver_timer + 1) % (100 * screensaver_tbs);

  return;

}
