//------------------------------------------------------------------------------
//                         Copyright 2003, P. Hulshoff
//                               The Netherlands
//------------------------------------------------------------------------------
//    This file is part of MAMED.
//
//    MAMED is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MAMED is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MAMED; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                                Includes
//------------------------------------------------------------------------------


#include <iostream>
#include "classes.h"
#include "fileio.h"
#include "gui.h"
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
using namespace std;


//------------------------------------------------------------------------------
//                            Global variables
//------------------------------------------------------------------------------


bool next_game_pressed = false;
bool prev_game_pressed = false;
bool screensaver       = false;
int  no_active_io      = 0;
bool game_played       = false;
bool help_key_pressed  = false;


//------------------------------------------------------------------------------
//                           External variables
//------------------------------------------------------------------------------


extern bool         play_music;
extern int          nr_music_played;
extern game        *current_game;
extern int          nr_games;
extern Mix_Music   *music;
extern string       mame_exec;
extern order_type   order;
extern int          key_favourites;
extern int          key_music;
extern int          key_order;
extern int          key_play;
extern int          key_quit;
extern int          key_shot;
extern int          key_screendump;
extern int          key_next_game;
extern int          key_next_letter;
extern int          key_next_section;
extern int          key_prev_game;
extern int          key_prev_letter;
extern int          key_prev_section;
extern int          key_help;
extern display_type shot;
extern display_type show_shot;
extern int          screensaver_tba;
extern int          screensaver_timer;
extern SDL_Surface *screen;
extern int          screenshotx;
extern int          screenshoty;
extern string       games_font;
extern int          games_red;
extern int          games_green;
extern int          games_blue;
extern int          games_sel_red;
extern int          games_sel_green;
extern int          games_sel_blue;
extern string       catver;
extern string       fadeout;


//------------------------------------------------------------------------------
//                                Functions
//------------------------------------------------------------------------------


// Read the joystick input, and update the current game/music accordingly
void handle_input( )
{
  SDL_Event test_event;
  no_active_io++;
  while ( SDL_PollEvent( &test_event ) )
  {
    if ( test_event.type == SDL_KEYUP )
    {
      if ( test_event.key.keysym.sym == key_next_game ) // next game
        next_game_pressed = false;
      if ( test_event.key.keysym.sym == key_prev_game ) // previous game
        prev_game_pressed = false;
      if ( test_event.key.keysym.sym == key_help )      // display help image
        help_key_pressed = false;
    }
    if ( test_event.type == SDL_KEYDOWN )
    {
      screensaver = false;
      no_active_io = 0;
      if ( test_event.key.keysym.sym == key_help ) // display help image
        help_key_pressed = true;
      if ( test_event.key.keysym.sym == key_next_game ) // next game
        next_game_pressed = true;
      if ( test_event.key.keysym.sym == key_prev_game ) // previous game
        prev_game_pressed = true;
      if ( test_event.key.keysym.sym == key_next_letter ) // next letter
      {
        while ( ((*current_game).get_name( ))[0] == ((*((*current_game).get_next( ))).get_name( ))[0] )
          current_game = (*current_game).get_next( );
        current_game = (*current_game).get_next( );
        // Set current_game to an available game
        while ( !(*current_game).is_available( ) )
          current_game = (*current_game).get_next( );
      }
      if ( test_event.key.keysym.sym == key_prev_letter ) // previous letter
      {
        game *tmp_game = current_game;
        while ( ((*current_game).get_name( ))[0] == ((*((*current_game).get_prev( ))).get_name( ))[0] )
          current_game = (*current_game).get_prev( );
        current_game = (*current_game).get_prev( );
        while ( ((*current_game).get_name( ))[0] == ((*((*current_game).get_prev( ))).get_name( ))[0] )
          current_game = (*current_game).get_prev( );
        // Set current_game to an available game
        while ( !(*current_game).is_available( ) )
          current_game = (*current_game).get_next( );
        if ( tmp_game == current_game )
          while ( !(*(*current_game).get_prev( )).is_available( ) )
            current_game = (*current_game).get_prev( );
      }
      if ( test_event.key.keysym.sym == key_next_section ) // next section
      {
        if ( order == year )
        {
          while ( (*current_game).get_year( ) == ((*((*current_game).get_next( ))).get_year( )) )
            current_game = (*current_game).get_next( );
          current_game = (*current_game).get_next( );
        }
        else if ( order == version )
        {
          while ( (*current_game).get_version( ) == ((*((*current_game).get_next( ))).get_version( )) )
            current_game = (*current_game).get_next( );
          current_game = (*current_game).get_next( );
        }
        else if ( order == category )
        {
          string::size_type pos;
          string cur_cat  = (*current_game).get_cat( );
          string next_cat = ((*((*current_game).get_next( ))).get_cat( ));
          if ( catver == "stripped" )
          {
            if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              cur_cat = cur_cat.substr( 0, pos-1 );
            if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              cur_cat = cur_cat.substr( 0, pos );
            if ( (pos  = next_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              next_cat = next_cat.substr( 0, pos-1 );
            if ( (pos  = next_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              next_cat = next_cat.substr( 0, pos );
          }
          while ( cur_cat == next_cat )
          {
            current_game = (*current_game).get_next( );
            cur_cat  = (*current_game).get_cat( );
            next_cat = ((*((*current_game).get_next( ))).get_cat( ));
            if ( catver == "stripped" )
            {
              if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                cur_cat = cur_cat.substr( 0, pos-1 );
              if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                cur_cat = cur_cat.substr( 0, pos );
              if ( (pos  = next_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                next_cat = next_cat.substr( 0, pos-1 );
              if ( (pos  = next_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                next_cat = next_cat.substr( 0, pos );
            }
          }
          current_game = (*current_game).get_next( );
        }
        // Set current_game to an available game
        while ( !(*current_game).is_available( ) )
          current_game = (*current_game).get_next( );
      }
      if ( test_event.key.keysym.sym == key_prev_section ) // previous section
      {
        game *tmp_game = current_game;
        if( order == year )
        {
          while ( ((*current_game).get_year( )) == ((*((*current_game).get_prev( ))).get_year( )) )
            current_game = (*current_game).get_prev( );
          current_game = (*current_game).get_prev( );
          while ( ((*current_game).get_year( )) == ((*((*current_game).get_prev( ))).get_year( )) )
            current_game = (*current_game).get_prev( );
        }
        else if ( order == version )
        {
          while ( ((*current_game).get_version( )) == ((*((*current_game).get_prev( ))).get_version( )) )
            current_game = (*current_game).get_prev( );
          current_game = (*current_game).get_prev( );
          while ( ((*current_game).get_version( )) == ((*((*current_game).get_prev( ))).get_version( )) )
            current_game = (*current_game).get_prev( );
        }
        else if ( order == category )
        {
          string::size_type pos;
          string cur_cat  = (*current_game).get_cat( );
          string prev_cat = ((*((*current_game).get_prev( ))).get_cat( ));
          if ( catver == "stripped" )
          {
            if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              cur_cat = cur_cat.substr( 0, pos-1 );
            if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              cur_cat = cur_cat.substr( 0, pos );
            if ( (pos  = prev_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              prev_cat = prev_cat.substr( 0, pos-1 );
            if ( (pos  = prev_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              prev_cat = prev_cat.substr( 0, pos );
          }
          while ( cur_cat == prev_cat )
          {
            current_game = (*current_game).get_prev( );
            cur_cat  = (*current_game).get_cat( );
            prev_cat = ((*((*current_game).get_prev( ))).get_cat( ));
            if ( catver == "stripped" )
            {
              if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                cur_cat = cur_cat.substr( 0, pos-1 );
              if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                cur_cat = cur_cat.substr( 0, pos );
              if ( (pos  = prev_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                prev_cat = prev_cat.substr( 0, pos-1 );
              if ( (pos  = prev_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                prev_cat = prev_cat.substr( 0, pos );
            }
          }
          current_game = (*current_game).get_prev( );
          cur_cat  = (*current_game).get_cat( );
          prev_cat = ((*((*current_game).get_prev( ))).get_cat( ));
          if ( catver == "stripped" )
          {
            if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              cur_cat = cur_cat.substr( 0, pos-1 );
            if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              cur_cat = cur_cat.substr( 0, pos );
            if ( (pos  = prev_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
              prev_cat = prev_cat.substr( 0, pos-1 );
            if ( (pos  = prev_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
              prev_cat = prev_cat.substr( 0, pos );
          }
          while ( cur_cat == prev_cat )
          {
            current_game = (*current_game).get_prev( );
            cur_cat  = (*current_game).get_cat( );
            prev_cat = ((*((*current_game).get_prev( ))).get_cat( ));
            if ( catver == "stripped" )
            {
              if ( (pos = cur_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                cur_cat = cur_cat.substr( 0, pos-1 );
              if ( (pos = cur_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                cur_cat = cur_cat.substr( 0, pos );
              if ( (pos  = prev_cat.find( "/", 0 )) != string::npos ) // Remove extentions on the filetype
                prev_cat = prev_cat.substr( 0, pos-1 );
              if ( (pos  = prev_cat.find( " *Mature*" )) != string::npos ) // Remove *Mature* extention
                prev_cat = prev_cat.substr( 0, pos );
            }
          }
        }
        // Set current_game to an available game
        while ( !(*current_game).is_available( ) )
          current_game = (*current_game).get_next( );
        if ( tmp_game == current_game )
          while ( !(*(*current_game).get_prev( )).is_available( ) )
            current_game = (*current_game).get_prev( );
      }
      if ( test_event.key.keysym.sym == key_order ) // switch order
      {
        switch ( order )
        {
          case year:     order = version;break;
          case version:  order = none;break;
          case none:     order = category;break;
          case category: order = year;break;
        }
        sort( current_game, nr_games );
      }
      if ( test_event.key.keysym.sym == key_shot ) // switch displayed shot
      {
        switch ( shot )
        {
          case screenshot: shot = titleshot;break;
          case titleshot:  shot = flyer;break;
          case flyer:      shot = cabinet;break;
          case cabinet:    shot = marquee;break;
          case marquee:    shot = cycle;break;
          case cycle:      shot = screenshot;break;
        }
        if ( shot != cycle )
          show_shot = shot;
      }
      if ( test_event.key.keysym.sym == key_play )
      {
        SDL_Color color = {255, 255, 255, 0};
        print_string( "Loading...", screen, games_font, screen->w/15, screenshotx, screenshoty, ALIGN_CENTER, ALIGN_CENTER, color );
        SDL_UpdateRect( screen, 0, 0, 0, 0 );
        if ( play_music && (music != NULL) )
          if ( fadeout == "on" )
            Mix_FadeOutMusic( 5000 );
          else
            Mix_CloseAudio( );
        string run_game;
        if ( (*current_game).get_exec( ) == "" )
          run_game = mame_exec + " " + (*current_game).get_file( );
        else
          run_game = (*current_game).get_exec( );
        system( run_game.c_str( ) ); 
        game_played = true;

        // Clear eventlist
        SDL_Event clear_event;
        while ( SDL_PollEvent( &clear_event ) );

        nr_music_played = 0;
        if ( play_music && (music != NULL) )
        {
          if ( fadeout == "on" )
            Mix_PlayMusic( music, 1 );
          else
          {
            int    audio_rate     = 44100;
            Uint16 audio_format   = AUDIO_S16;
            int    audio_channels = 2;
            int    audio_buffers  = 4096;
            if ( Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0 )
            {
              cout << "Cannot open audio: " << SDL_GetError( ) << endl;
              play_music = false;
            }
            else
              Mix_PlayMusic( music, 1 );
          }
        }
      }
      if ( test_event.key.keysym.sym == key_music )
        if ( play_music && (music != NULL) )
        {
          play_music = false;
          Mix_HaltMusic( );
        }
        else if ( music != NULL )
        {
          play_music      = true;
          nr_music_played = 0;
          Mix_PlayMusic( music, 1 );
        }
      if ( test_event.key.keysym.sym == key_screendump )
        system( "/usr/X11R6/bin/xwd -root > screendump" );
      if ( test_event.key.keysym.sym == key_favourites )
      {
        if ( (*current_game).get_cat( ) == "Favourites" )
          (*current_game).set_cat( (*current_game).get_orig_cat( ) );
        else
          (*current_game).set_cat( "Favourites" );
        sort( current_game, nr_games );
      }
      if ( test_event.key.keysym.sym == key_quit )
      {
        bool chosen = false;
        bool quit   = false;
        while ( !chosen )
        {
          SDL_Color color  = {games_red,     games_green,     games_blue,     0};
          SDL_Color scolor = {games_sel_red, games_sel_green, games_sel_blue, 0};
          if ( !quit )
          {
            print_string( "Continue",    screen, games_font, screen->w/30, screenshotx, screenshoty-screen->w/60, ALIGN_CENTER, ALIGN_CENTER, scolor );
            print_string( "Close MAMEd", screen, games_font, screen->w/30, screenshotx, screenshoty+screen->w/60, ALIGN_CENTER, ALIGN_CENTER, color );
            SDL_UpdateRect( screen, 0, 0, 0, 0 );
          }
          else
          {
            print_string( "Continue",    screen, games_font, screen->w/30, screenshotx, screenshoty-screen->w/60, ALIGN_CENTER, ALIGN_CENTER, color );
            print_string( "Close MAMEd", screen, games_font, screen->w/30, screenshotx, screenshoty+screen->w/60, ALIGN_CENTER, ALIGN_CENTER, scolor );
            SDL_UpdateRect( screen, 0, 0, 0, 0 );
          }
          if ( SDL_PollEvent( &test_event ) )
            if ( test_event.type == SDL_KEYDOWN )
              if ( ( test_event.key.keysym.sym == key_next_game ) ||
                   ( test_event.key.keysym.sym == key_prev_game ) )
                quit = !quit;
              else
                chosen = true;
          SDL_Delay( 10 );
        }
        if ( quit )
        {
          write_mamedgamelistfile( );
          Mix_CloseAudio( );
          SDL_Quit( );
          exit( 0 );
        }
        game_played = true;
      }
    }
  }

  if ( next_game_pressed )
  {
    screensaver  = false;
    no_active_io = 0;
    current_game = (*current_game).get_next( );
    // Set current_game to an available game
    while ( !(*current_game).is_available( ) )
      current_game = (*current_game).get_next( );
  }
  if ( prev_game_pressed )
  {
    screensaver  = false;
    no_active_io = 0;
    current_game = (*current_game).get_prev( );
    // Set current_game to an available game
    while ( !(*current_game).is_available( ) )
      current_game = (*current_game).get_prev( );
  }

  if ( ( no_active_io == 100 * screensaver_tba ) &&
       ( screensaver_tba != 0 ) )
  {
    screensaver       = true;
    no_active_io      = 0;
    screensaver_timer = 0;
  }

  if ( shot == cycle && no_active_io%100 == 0 )
    switch ( show_shot )
    {
      case screenshot: show_shot = titleshot;break;
      case titleshot:  show_shot = flyer;break;
      case flyer:      show_shot = cabinet;break;
      case cabinet:    show_shot = marquee;break;
      case marquee:    show_shot = screenshot;break;
      case cycle:      show_shot = screenshot;break;
    }

  return;

}
