//------------------------------------------------------------------------------
//                         Copyright 2003, P. Hulshoff
//                               The Netherlands
//------------------------------------------------------------------------------
//    This file is part of MAMED.
//
//    MAMED is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    MAMED is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with MAMED; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                                Includes
//------------------------------------------------------------------------------


#include <iostream>
#include <time.h>
#include <vector>
#include <stdlib.h>
#include "SDL/SDL.h"
#include "SDL/SDL_mixer.h"
#include "classes.h"
#include "fileio.h"
#include "gui.h"
#include "io.h"
using namespace std;


//------------------------------------------------------------------------------
//                            Global variables
//------------------------------------------------------------------------------


Mix_Music    *music;
int          nr_music_played;


//------------------------------------------------------------------------------
//                           External variables
//------------------------------------------------------------------------------


extern bool            play_music;
extern string          music_dir;
extern bool            music_repeat;
extern int             nr_music;
extern vector<string>  music_names;
extern SDL_Surface    *screen;
extern bool            screensaver;


//------------------------------------------------------------------------------
//                              Main program
//------------------------------------------------------------------------------


int main()
{
  // Initialise pseudo random generator
  srand( time( NULL ) );

  // Initialised nr of music files played
  nr_music_played = 0;

  // Initialise MAMED
  read_config( );
  init_sdl( );
  read_gamelist( );

  // MAMED main loop
  while ( true )
  {
    SDL_Delay( 10 ); // Don't hog the CPU
    handle_input( );
    if ( screensaver )
      run_screensaver( );
    else
      update_screen( );
    if ( music_names.size( ) != 0 &&
         !Mix_PlayingMusic( )     &&
         play_music               &&
        (nr_music_played < nr_music) )
    {
      Mix_FreeMusic( music );
      int music_sel = rand( )%music_names.size( );
      music = Mix_LoadMUS( (music_dir + "/" + music_names[music_sel]).c_str( ) );
      if ( music != NULL )
        Mix_PlayMusic( music, 1 );
      else
        cout << "Could not load music file: " << music_dir << "/" << music_names[music_sel] << endl;
      if ( !music_repeat )
        nr_music_played++;
    }
  }

  return 0;

}
